from __future__ import annotations

import contextlib
import logging
import shutil

from agents import set_tracing_disabled

from .agent_runtime import build_agent
from .chat_service import ChatService
from .config import Settings, apply_runtime_environment
from .db import ChatHistoryStore
from .mcp_runtime import MCPRuntime
from .ui import build_demo


LOGGER = logging.getLogger("openkite")


def _configure_logging() -> None:
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(levelname)s [%(name)s] %(message)s",
    )


def _validate_runtime_dependencies() -> None:
    if shutil.which("npx") is None:
        raise RuntimeError(
            "npx was not found in PATH. Install Node.js before starting openkite."
        )


def run_chat_app(settings: Settings) -> None:
    """Start the OpenKite Gradio chat application."""
    _configure_logging()
    set_tracing_disabled(True)
    apply_runtime_environment(settings)
    _validate_runtime_dependencies()

    store = ChatHistoryStore(settings.db_path)
    store.initialize()
    LOGGER.info("Chat history DB ready: %s", settings.db_path)

    runtime = MCPRuntime(settings)
    agent = build_agent(settings, runtime)
    chat_service = ChatService(agent, store)
    demo = build_demo(chat_service)

    @contextlib.asynccontextmanager
    async def _lifespan(app):
        await runtime.connect()
        LOGGER.info("Connected to Kite and Firecrawl MCP servers.")
        yield
        await runtime.close()
        LOGGER.info("MCP runtime closed.")

    demo.launch(
        server_name=settings.host,
        server_port=settings.port,
        share=settings.share,
        app_kwargs={"lifespan": _lifespan},
    )



from __future__ import annotations

import os
import subprocess

from agents.mcp import MCPServerSse, MCPServerStdio
from mcp.client.stdio import stdio_client

from .config import Settings


class SessionMCPServerSse(MCPServerSse):
    """SSE MCP server that stays connected for the whole app session."""

    async def cleanup(self) -> None:
        # Runner triggers cleanup around each run; suppress to keep login session alive.
        return

    async def __aexit__(self, *args) -> None:
        return

    async def close_session(self) -> None:
        await super().cleanup()


class SessionMCPServerStdio(MCPServerStdio):
    """Stdio MCP server that keeps subprocess alive between chat turns."""

    def create_streams(self):
        return stdio_client(self.params, errlog=subprocess.DEVNULL)

    async def cleanup(self) -> None:
        return

    async def __aexit__(self, *args) -> None:
        return

    async def close_session(self) -> None:
        await super().cleanup()


class MCPRuntime:
    """Holds and manages all MCP connections used by OpenKite."""

    def __init__(self, settings: Settings) -> None:
        self.kite_server = SessionMCPServerSse(
            name="kite",
            params={"url": settings.kite_sse_url},
            cache_tools_list=True,
            client_session_timeout_seconds=30,
        )
        self.firecrawl_server = SessionMCPServerStdio(
            name="firecrawl",
            params={
                "command": "npx",
                "args": ["-y", "firecrawl-mcp"],
                "env": {
                    **os.environ.copy(),
                    "FIRECRAWL_API_KEY": settings.firecrawl_api_key,
                },
            },
            cache_tools_list=True,
            client_session_timeout_seconds=30,
        )
        self._connected = False

    @property
    def servers(self) -> list:
        return [self.kite_server, self.firecrawl_server]

    async def connect(self) -> None:
        if self._connected:
            return
        await self.kite_server.connect()
        await self.firecrawl_server.connect()
        self._connected = True

    async def close(self) -> None:
        if not self._connected:
            return
        await self.firecrawl_server.close_session()
        await self.kite_server.close_session()
        self._connected = False



from __future__ import annotations

import html
import json
import time
import uuid

import gradio as gr
from agents import Runner

from .db import ChatHistoryStore


TOOL_BLOCK_START = "<!--OPENKITE_TOOL_CALLS_START-->"
TOOL_BLOCK_END = "<!--OPENKITE_TOOL_CALLS_END-->"
FOOTER_BLOCK_START = "<!--OPENKITE_UI_FOOTER_START-->"
FOOTER_BLOCK_END = "<!--OPENKITE_UI_FOOTER_END-->"


def _strip_wrapped_block(text: str, start_marker: str, end_marker: str) -> str:
    while True:
        start = text.find(start_marker)
        if start == -1:
            break
        end = text.find(end_marker, start + len(start_marker))
        if end == -1:
            text = text[:start]
            break
        text = text[:start] + text[end + len(end_marker):]
    return text


def _strip_ui_metadata(text: str) -> str:
    cleaned = _strip_wrapped_block(text, TOOL_BLOCK_START, TOOL_BLOCK_END)
    cleaned = _strip_wrapped_block(cleaned, FOOTER_BLOCK_START, FOOTER_BLOCK_END)
    return cleaned.strip()


def _extract_text(content) -> str:
    if isinstance(content, str):
        return content
    if isinstance(content, list):
        parts: list[str] = []
        for item in content:
            if isinstance(item, dict):
                parts.append(item.get("text") or item.get("value") or "")
            else:
                parts.append(str(item))
        return "".join(parts)
    return str(content)


def _build_input(history: list[dict], message: str) -> list[dict]:
    messages: list[dict] = []
    for turn in history:
        role = turn.get("role", "")
        if role not in {"user", "assistant"}:
            continue
        text = _extract_text(turn.get("content", ""))
        if role == "assistant":
            text = _strip_ui_metadata(text)
        if text:
            messages.append({"role": role, "content": text})
    messages.append({"role": "user", "content": message})
    return messages


def _render_tool_call_dropdown(name: str, args: str, status: str) -> str:
    icon = "⏳" if status == "running" else "✅"
    label = "In progress" if status == "running" else "Completed"
    safe_name = html.escape(name)
    safe_args = html.escape(args) if args else "No arguments"
    return f"<li>{icon} <strong>{safe_name}</strong> - {label}<br/><pre>{safe_args}</pre></li>"


def _render_tool_calls_container(tool_calls: list[tuple[str, str]], include_running: bool) -> str:
    if not tool_calls:
        return ""

    items: list[str] = []
    for index, (name, args) in enumerate(tool_calls):
        status = "done"
        if include_running and index == len(tool_calls) - 1:
            status = "running"
        items.append(_render_tool_call_dropdown(name, args, status))

    running_suffix = " (running)" if include_running else ""
    open_attr = " open" if include_running else ""
    return (
        f"<details{open_attr}>"
        f"<summary>🛠️ Tool calls used ({len(tool_calls)}){running_suffix}</summary>"
        f"<ul>{''.join(items)}</ul>"
        f"</details>"
    )


def _tool_status_block(tool_calls: list[tuple[str, str]]) -> str:
    body = _render_tool_calls_container(tool_calls, include_running=True)
    return f"Agent is working...\n\n{body}"


def _tool_calls_block(tool_calls: list[tuple[str, str]]) -> str:
    body = _render_tool_calls_container(tool_calls, include_running=False)
    if not body:
        return ""
    return f"{TOOL_BLOCK_START}\n{body}\n{TOOL_BLOCK_END}\n\n"


class ChatService:
    def __init__(self, agent, store: ChatHistoryStore) -> None:
        self._agent = agent
        self._store = store

    def history_choices(self) -> list[tuple[str, str]]:
        return [(summary.title, summary.session_id) for summary in self._store.list_conversations()]

    def load_history(self, session_id: str) -> list[dict]:
        return self._store.load_conversation(session_id)

    async def chat(self, message: str, history: list[dict], session_id: str = ""):
        start = time.perf_counter()
        history = history or []
        input_messages = _build_input(history, message)

        tool_calls: list[tuple[str, str]] = []
        yield ("Thinking...", session_id, gr.update())

        final_output = ""
        try:
            result = Runner.run_streamed(self._agent, input_messages)
            async for event in result.stream_events():
                    if event.type == "run_item_stream_event" and event.name == "tool_called":
                        raw = event.item.raw_item
                        tool_name = getattr(raw, "name", None) or "tool"
                        raw_args = getattr(raw, "arguments", "") or ""

                        try:
                            parsed_args = json.loads(raw_args)
                            args_str = ", ".join(f"{k}={v}" for k, v in parsed_args.items())
                        except Exception:
                            args_str = str(raw_args)

                        if len(args_str) > 80:
                            args_str = args_str[:77] + "..."

                        tool_calls.append((tool_name, args_str))
                        yield (_tool_status_block(tool_calls), session_id, gr.update())

            final_output = str(result.final_output or "")
        except Exception as exc:
            final_output = f"Request failed: {exc}"

        elapsed = time.perf_counter() - start
        footer = (
            f"\n\n{FOOTER_BLOCK_START}\n"
            f"---\n"
            f"Response time: {elapsed:.1f}s\n"
            f"{FOOTER_BLOCK_END}"
        )
        final_text = _tool_calls_block(tool_calls) + final_output + footer

        if not session_id:
            session_id = str(uuid.uuid4())

        full_history = input_messages + [{"role": "assistant", "content": final_text}]
        self._store.save_conversation(session_id, full_history)

        yield (final_text, session_id, gr.update(choices=self.history_choices(), value=session_id))



from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
import os

from dotenv import load_dotenv


DEFAULT_MODEL = "gpt-5.4"
DEFAULT_KITE_SSE_URL = "https://mcp.kite.trade/sse"
DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 7860
DEFAULT_DB_PATH = Path.home() / ".openkite" / "chat_history.db"


class SettingsError(ValueError):
    """Raised when required runtime configuration is missing or invalid."""


@dataclass(frozen=True)
class Settings:
    openai_api_key: str
    firecrawl_api_key: str
    model: str
    kite_sse_url: str
    db_path: Path
    host: str
    port: int
    share: bool


def _required_env(name: str) -> str:
    value = os.environ.get(name, "").strip()
    if not value:
        raise SettingsError(f"Missing required environment variable: {name}")
    return value


def _parse_port(raw_port: str) -> int:
    try:
        port = int(raw_port)
    except ValueError as exc:
        raise SettingsError(f"OPENKITE_PORT must be an integer, got: {raw_port}") from exc

    if not 1 <= port <= 65535:
        raise SettingsError(f"OPENKITE_PORT must be between 1 and 65535, got: {port}")
    return port


def load_settings(env_file: str | None = None) -> Settings:
    """Load settings from .env and process environment variables."""
    load_dotenv(dotenv_path=env_file, override=False)

    openai_api_key = _required_env("OPENAI_API_KEY")
    firecrawl_api_key = _required_env("FIRECRAWL_API_KEY")

    model = os.environ.get("OPENKITE_MODEL", DEFAULT_MODEL).strip() or DEFAULT_MODEL
    kite_sse_url = (
        os.environ.get("ZERODHA_MCP_SSE_URL", DEFAULT_KITE_SSE_URL).strip()
        or DEFAULT_KITE_SSE_URL
    )

    db_path_value = os.environ.get("OPENKITE_DB_PATH", "").strip()
    db_path = Path(db_path_value).expanduser() if db_path_value else DEFAULT_DB_PATH
    db_path.parent.mkdir(parents=True, exist_ok=True)

    host = os.environ.get("OPENKITE_HOST", DEFAULT_HOST).strip() or DEFAULT_HOST
    port = _parse_port(os.environ.get("OPENKITE_PORT", str(DEFAULT_PORT)).strip())

    share_raw = os.environ.get("OPENKITE_SHARE", "false").strip().lower()
    share = share_raw in {"1", "true", "yes", "on"}

    return Settings(
        openai_api_key=openai_api_key,
        firecrawl_api_key=firecrawl_api_key,
        model=model,
        kite_sse_url=kite_sse_url,
        db_path=db_path,
        host=host,
        port=port,
        share=share,
    )


def apply_runtime_environment(settings: Settings) -> None:
    """Ensure downstream libraries see required credentials in process env."""
    os.environ["OPENAI_API_KEY"] = settings.openai_api_key
    os.environ["FIRECRAWL_API_KEY"] = settings.firecrawl_api_key



from __future__ import annotations

from openkite.ui import TITLE_BANNER_HTML, build_demo


class _StubChatService:
    def history_choices(self) -> list[tuple[str, str]]:
        return []

    def load_history(self, session_id: str) -> list[dict]:
        return []

    async def chat(self, message: str, history: list[dict], session_id: str = ""):
        yield ("", session_id, None)


def test_title_banner_uses_self_contained_logo_image() -> None:
    assert '<img id="title-logo"' in TITLE_BANNER_HTML
    assert "data:image/svg+xml," in TITLE_BANNER_HTML
    assert ".cls-1" not in TITLE_BANNER_HTML
    assert ".cls-2" not in TITLE_BANNER_HTML


def test_build_demo_renders_title_banner_html() -> None:
    app = build_demo(_StubChatService())

    html_values = [
        component.get("props", {}).get("value", "")
        for component in app.config.get("components", [])
        if component.get("type") == "html"
    ]

    assert TITLE_BANNER_HTML in html_values


from __future__ import annotations

import pytest

from openkite.config import SettingsError, load_settings


REQUIRED_ENV = [
    "OPENAI_API_KEY",
    "FIRECRAWL_API_KEY",
    "OPENKITE_MODEL",
    "ZERODHA_MCP_SSE_URL",
    "OPENKITE_DB_PATH",
    "OPENKITE_HOST",
    "OPENKITE_PORT",
    "OPENKITE_SHARE",
]


def _clear_env(monkeypatch: pytest.MonkeyPatch) -> None:
    for name in REQUIRED_ENV:
        monkeypatch.delenv(name, raising=False)


def test_load_settings_success(monkeypatch: pytest.MonkeyPatch, tmp_path) -> None:
    _clear_env(monkeypatch)

    monkeypatch.setenv("OPENAI_API_KEY", "sk-test")
    monkeypatch.setenv("FIRECRAWL_API_KEY", "fc-test")
    monkeypatch.setenv("OPENKITE_MODEL", "gpt-5.4")
    monkeypatch.setenv("ZERODHA_MCP_SSE_URL", "https://mcp.kite.trade/sse")
    monkeypatch.setenv("OPENKITE_DB_PATH", str(tmp_path / "chat_history.db"))
    monkeypatch.setenv("OPENKITE_HOST", "127.0.0.1")
    monkeypatch.setenv("OPENKITE_PORT", "7860")
    monkeypatch.setenv("OPENKITE_SHARE", "false")

    settings = load_settings()

    assert settings.openai_api_key == "sk-test"
    assert settings.firecrawl_api_key == "fc-test"
    assert settings.model == "gpt-5.4"
    assert settings.port == 7860
    assert settings.share is False


def test_missing_required_env(monkeypatch: pytest.MonkeyPatch) -> None:
    _clear_env(monkeypatch)

    with pytest.raises(SettingsError):
        load_settings()



from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime, timezone
from pathlib import Path
import sqlite3


@dataclass(frozen=True)
class ConversationSummary:
    session_id: str
    title: str


class ChatHistoryStore:
    """SQLite-backed persistence for chat conversations."""

    def __init__(self, db_path: Path) -> None:
        self._db_path = db_path

    def initialize(self) -> None:
        con = sqlite3.connect(self._db_path)
        try:
            con.execute(
                """
                CREATE TABLE IF NOT EXISTS conversations (
                    id TEXT PRIMARY KEY,
                    title TEXT NOT NULL,
                    created_at TEXT NOT NULL,
                    updated_at TEXT NOT NULL
                )
                """
            )
            con.execute(
                """
                CREATE TABLE IF NOT EXISTS messages (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    conversation_id TEXT NOT NULL,
                    role TEXT NOT NULL,
                    content TEXT NOT NULL,
                    ts TEXT NOT NULL
                )
                """
            )
            con.commit()
        finally:
            con.close()

    @staticmethod
    def _title_from_history(history: list[dict]) -> str:
        last_user_content = ""
        for message in history:
            if message.get("role") == "user":
                last_user_content = str(message.get("content", ""))

        normalized = " ".join(last_user_content.split())
        if not normalized:
            return "Untitled"

        words = normalized.split(" ")
        if len(words) <= 10:
            return normalized
        return f"{' '.join(words[:10])}..."

    def save_conversation(self, session_id: str, history: list[dict]) -> None:
        if not session_id or not history:
            return

        now = datetime.now(timezone.utc).isoformat()
        title = self._title_from_history(history)

        con = sqlite3.connect(self._db_path)
        try:
            row = con.execute("SELECT id FROM conversations WHERE id = ?", (session_id,)).fetchone()

            if row:
                con.execute(
                    "UPDATE conversations SET title = ?, updated_at = ? WHERE id = ?",
                    (title, now, session_id),
                )
            else:
                con.execute(
                    "INSERT INTO conversations (id, title, created_at, updated_at) VALUES (?, ?, ?, ?)",
                    (session_id, title, now, now),
                )

            con.execute("DELETE FROM messages WHERE conversation_id = ?", (session_id,))
            con.executemany(
                "INSERT INTO messages (conversation_id, role, content, ts) VALUES (?, ?, ?, ?)",
                [(session_id, m.get("role", ""), m.get("content", ""), now) for m in history],
            )
            con.commit()
        finally:
            con.close()

    def list_conversations(self, limit: int = 100) -> list[ConversationSummary]:
        con = sqlite3.connect(self._db_path)
        try:
            rows = con.execute(
                "SELECT id, title FROM conversations ORDER BY updated_at DESC LIMIT ?",
                (limit,),
            ).fetchall()
            return [ConversationSummary(session_id=row[0], title=row[1]) for row in rows]
        finally:
            con.close()

    def load_conversation(self, session_id: str) -> list[dict]:
        if not session_id:
            return []

        con = sqlite3.connect(self._db_path)
        try:
            rows = con.execute(
                "SELECT role, content FROM messages WHERE conversation_id = ? ORDER BY id",
                (session_id,),
            ).fetchall()
            return [{"role": row[0], "content": row[1]} for row in rows]
        finally:
            con.close()



from __future__ import annotations

import argparse
from dataclasses import replace

from .app import run_chat_app
from .config import Settings, SettingsError, load_settings


def _build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="openkite",
        description="OpenKite trading assistant",
    )
    parser.add_argument(
        "command",
        nargs="?",
        default="chat",
        choices=["chat", "check"],
        help="chat: launch UI, check: validate configuration",
    )
    parser.add_argument(
        "--env-file",
        default=None,
        help="Path to .env file (default: auto-discover .env in working directory)",
    )
    parser.add_argument("--host", default=None, help="Override OPENKITE_HOST")
    parser.add_argument("--port", type=int, default=None, help="Override OPENKITE_PORT")
    parser.add_argument(
        "--share",
        action="store_true",
        help="Override OPENKITE_SHARE and request a public Gradio share link",
    )
    return parser


def _apply_overrides(settings: Settings, args: argparse.Namespace) -> Settings:
    overrides = {}
    if args.host:
        overrides["host"] = args.host
    if args.port:
        overrides["port"] = args.port
    if args.share:
        overrides["share"] = True
    if overrides:
        return replace(settings, **overrides)
    return settings


def main(argv: list[str] | None = None) -> int:
    parser = _build_parser()
    args = parser.parse_args(argv)

    try:
        settings = load_settings(args.env_file)
        settings = _apply_overrides(settings, args)
    except SettingsError as exc:
        parser.exit(2, f"Configuration error: {exc}\n")

    if args.command == "check":
        print("Configuration loaded successfully.")
        print(f"Model: {settings.model}")
        print(f"Kite SSE URL: {settings.kite_sse_url}")
        print(f"DB Path: {settings.db_path}")
        print(f"Host: {settings.host}:{settings.port}")
        print("OPENAI_API_KEY: set")
        print("FIRECRAWL_API_KEY: set")
        return 0

    try:
        run_chat_app(settings)
    except RuntimeError as exc:
        parser.exit(2, f"Startup error: {exc}\n")

    return 0


def entrypoint() -> int:
    return main()



from __future__ import annotations

from importlib import resources

from agents import Agent

from .config import Settings
from .mcp_runtime import MCPRuntime
from .tools import screener_get_data


def _load_system_prompt() -> str:
    prompt_file = resources.files("openkite.prompts").joinpath("system_prompt.txt")
    return prompt_file.read_text(encoding="utf-8")


def build_agent(settings: Settings, runtime: MCPRuntime) -> Agent:
    """Create the OpenAI Agent instance for chat runs."""
    return Agent(
        name="Kite Assistant",
        model=settings.model,
        instructions=_load_system_prompt(),
        mcp_servers=runtime.servers,
        tools=[screener_get_data],
    )



from __future__ import annotations

from openkite.cli import main


def test_check_command(monkeypatch, tmp_path, capsys) -> None:
    monkeypatch.setenv("OPENAI_API_KEY", "sk-test")
    monkeypatch.setenv("FIRECRAWL_API_KEY", "fc-test")
    monkeypatch.setenv("OPENKITE_DB_PATH", str(tmp_path / "chat_history.db"))

    exit_code = main(["check"])

    captured = capsys.readouterr()
    assert exit_code == 0
    assert "Configuration loaded successfully." in captured.out



from __future__ import annotations

import uuid
from urllib.parse import quote

import gradio as gr

from .chat_service import ChatService


CUSTOM_CSS = """
.gradio-container {
    max-width: 100% !important;
    padding: 0 !important;
    background: #0b0e11 !important;
}
#title-banner {
    text-align: center;
    padding: 16px 0 10px 0;
    background: linear-gradient(135deg, #0b0e11 0%, #141a22 100%);
    border-bottom: 2px solid #00c87644;
}
#title-main {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    gap: 10px;
    flex-wrap: wrap;
}
#title-logo {
    display: block;
    width: 3.3rem;
    height: 2.2rem;
    flex: 0 0 3.3rem;
    object-fit: contain;
}
#title-banner h1 {
    font-size: 2.2rem !important;
    font-weight: 800 !important;
    margin: 0 !important;
    color: #00c876;
    letter-spacing: 1px;
}
#title-banner p {
    color: #8899aa;
    margin: 4px 0 0 0 !important;
    font-size: 0.95rem;
}
#sidebar {
    background: #0d1117 !important;
    border-right: 1px solid #1e2a38 !important;
    min-height: calc(100vh - 110px) !important;
}
#history-label {
    color: #8899aa;
    font-size: 1rem;
    margin: 12px 0 6px 4px !important;
}
#new-chat-btn {
    background: #00c876 !important;
    border-color: #00c876 !important;
    color: #0b0e11 !important;
    font-weight: 700 !important;
}
#new-chat-btn:hover {
    background: #00b86d !important;
    border-color: #00b86d !important;
}
footer {
    display: none !important;
}
"""


TITLE_LOGO_SVG = (
    '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 90 60" preserveAspectRatio="xMidYMid meet">'
    '<polygon fill="#2ecc71" points="30 0 0 30 30 60 60 30 90 0 30 0"/>'
    '<polygon fill="#27ae60" points="30 60 60 30 90 60 30 60"/>'
    '</svg>'
)
TITLE_LOGO_SRC = f"data:image/svg+xml,{quote(TITLE_LOGO_SVG, safe='')}"
TITLE_BANNER_HTML = (
    '<div id="title-banner">'
        '<div id="title-main">'
            f'<img id="title-logo" src="{TITLE_LOGO_SRC}" alt="" aria-hidden="true" />'
            '<h1>OpenKite - Trading Assistant</h1>'
        '</div>'
        '<p>Powered by Zerodha Kite MCP &bull; Firecrawl &bull; Screener.in</p>'
    '</div>'
)


EXAMPLES = [
    ["Login to Kite "],
    ["Show my current holdings with live prices, day change, unrealised P&L, and portfolio weights."],
    ["Analyze my portfolio and suggest which stocks I should consider selling"],
    ["Suggest 3 stocks I can buy today based on fundamentals and market momentum"],
    ["Show the last 4 quarters for HDFCBANK and tell me whether revenue, profit, and return ratios are improving or weakening."],
    ["Summarize the most important Reliance Industries news from the last 7 days and explain the likely market impact."],
    ["Which stocks are trending with high volume and strong fundamentals ?"],
    ["Review my top holdings by value using fundamentals and recent news and highlight key risks and opportunities"],
]


def build_demo(chat_service: ChatService) -> gr.Blocks:
    """Build and return the Gradio application."""

    def _on_new_chat():
        new_sid = str(uuid.uuid4())
        return [], [], new_sid, gr.update(choices=chat_service.history_choices(), value=None)

    def _on_load_conversation(selected_sid: str):
        if not selected_sid:
            return gr.update(), gr.update(), gr.update()
        messages = chat_service.load_history(selected_sid)
        return messages, messages, selected_sid

    session_id_state = gr.State(value="")

    with gr.Blocks(css=CUSTOM_CSS, title="OpenKite Trading Assistant", theme=gr.themes.Base()) as demo:
        gr.HTML(TITLE_BANNER_HTML)

        with gr.Row():
            with gr.Column(scale=1, min_width=230, elem_id="sidebar"):
                new_chat_btn = gr.Button("+ New Chat", elem_id="new-chat-btn")
                gr.HTML('<div id="history-label">History</div>')
                history_radio = gr.Radio(
                    choices=chat_service.history_choices(),
                    value=None,
                    label="",
                    show_label=False,
                    interactive=True,
                )

            with gr.Column(scale=4):
                chat_interface = gr.ChatInterface(
                    fn=chat_service.chat,
                    chatbot=gr.Chatbot(
                        elem_id="chatbot",
                        height="calc(100vh - 210px)",
                        buttons=["copy"],
                        render_markdown=True,
                    ),
                    additional_inputs=[session_id_state],
                    additional_inputs_accordion=gr.Accordion(visible=False),
                    additional_outputs=[session_id_state, history_radio],
                    examples=EXAMPLES,
                )

        demo.load(
            fn=lambda: gr.update(choices=chat_service.history_choices()),
            outputs=[history_radio],
        )

        new_chat_btn.click(
            fn=_on_new_chat,
            inputs=[],
            outputs=[chat_interface.chatbot, chat_interface.chatbot_state, session_id_state, history_radio],
            queue=False,
        )

        history_radio.change(
            fn=_on_load_conversation,
            inputs=[history_radio],
            outputs=[chat_interface.chatbot, chat_interface.chatbot_state, session_id_state],
            queue=False,
        )

    return demo



from __future__ import annotations

import json

from agents import function_tool
from click.testing import CliRunner


try:
    from screener_cli.cli import main as screener_main
except Exception:  # pragma: no cover
    screener_main = None


_screener_runner = CliRunner() if screener_main is not None else None


@function_tool
def screener_get_data(symbol: str, section: str, view: str = "consolidated") -> str:
    """Fetch financial data for an Indian stock from screener.in."""
    if _screener_runner is None or screener_main is None:
        return json.dumps(
            {
                "error": (
                    "screener_cli is not installed. Install a compatible package that exposes "
                    "screener_cli.cli:main to enable screener_get_data."
                )
            }
        )

    result = _screener_runner.invoke(screener_main, ["--view", view, symbol, section])
    if result.exit_code != 0:
        return json.dumps({"error": result.output.strip()})
    return result.output.strip()



from __future__ import annotations

from openkite.db import ChatHistoryStore


def test_db_roundtrip(tmp_path) -> None:
    db_path = tmp_path / "history.db"
    store = ChatHistoryStore(db_path)
    store.initialize()

    session_id = "session-1"
    history = [
        {"role": "user", "content": "hello"},
        {"role": "assistant", "content": "world"},
    ]

    store.save_conversation(session_id, history)

    summaries = store.list_conversations()
    assert summaries
    assert summaries[0].session_id == session_id

    loaded = store.load_conversation(session_id)
    assert loaded == history


def test_summary_title_uses_last_user_message_truncated_to_ten_words(tmp_path) -> None:
    db_path = tmp_path / "history.db"
    store = ChatHistoryStore(db_path)
    store.initialize()

    session_id = "session-2"
    history = [
        {"role": "user", "content": "first prompt"},
        {"role": "assistant", "content": "response"},
        {
            "role": "user",
            "content": "one two three four five six seven eight nine ten eleven twelve",
        },
    ]

    store.save_conversation(session_id, history)

    summaries = store.list_conversations()
    assert summaries
    assert summaries[0].title == "one two three four five six seven eight nine ten..."


def test_summary_title_updates_when_conversation_is_saved_again(tmp_path) -> None:
    db_path = tmp_path / "history.db"
    store = ChatHistoryStore(db_path)
    store.initialize()

    session_id = "session-3"
    first_history = [
        {"role": "user", "content": "initial question"},
        {"role": "assistant", "content": "initial answer"},
    ]
    store.save_conversation(session_id, first_history)

    second_history = [
        {"role": "user", "content": "initial question"},
        {"role": "assistant", "content": "initial answer"},
        {"role": "user", "content": "latest user message should be the title"},
        {"role": "assistant", "content": "latest answer"},
    ]
    store.save_conversation(session_id, second_history)

    summaries = store.list_conversations()
    assert summaries
    assert summaries[0].title == "latest user message should be the title"
